/* Frontend for libfsiclient 
* Copyright (c) 2004 Koen Kooi <koen@handhelds.org> 
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version
* 2 of the License, or (at your option) any later version.
*/


#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/time.h>
#include <string.h>

#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

#include "client/fsiclient.h"

#define BUF_SIZE  (FSI_FRAME_SIZE_BYTES * 512)

/*global vars, ugly, but it works*/
char fname[256];
GtkWidget*  img;
GtkWidget*	evil_label;

void write_pgm_file( unsigned char *buffer, int width, int height, FILE *fp )
{
	int x, y;
	
	fprintf( fp, "P5\n" );
	fprintf( fp, "%d %d\n", width, height );
	fprintf( fp, "255\n" );
	
	for( y = height; y > 0; y-- )
		for( x = width; x > 0; x-- )
			fprintf( fp, "%c", buffer[(x-1) + (y-1) * width] );
}




static gboolean delete_event( GtkWidget *widget,
                              GdkEvent  *event,
                              gpointer   data )
{
	gtk_main_quit ();
	return FALSE;
}

static void update_pic ( GtkWidget *widget,
						 gpointer   data )
{
	
	gtk_image_set_from_file(img, data);
	gtk_widget_show(img);
	GtkWidget* tmppic = gdk_pixbuf_new_from_file (data, NULL);
	gdk_pixbuf_save(tmppic, "test.jpg", "jpeg", NULL,
                 "quality", "80", NULL);
	
	gtk_label_set_text(evil_label, data);
	gtk_widget_show(evil_label);
	
	
}


static void scanfinger( GtkWidget *widget,
                      	gpointer   data )
{
   	
	/* copied from fsidumppgm.c */
	int result;
	void *buffer;
	unsigned int num_lines;
	unsigned int line_width;
	extern char fname[256];
	FILE *fp;
	struct fsi_context ctx;
	struct timeval tv_start;
	struct timeval tv_now;
	
	/* FIXME set a sane name */
	sprintf( fname, "fp_%08lx.pgm", time(NULL) );
	
	/* allocate buffer for fingerprint (any "large" buffer will do) */
	
	buffer = malloc( BUF_SIZE );
	if( buffer == NULL )
    {
		/* adapted to glib, FIXME, dialog */
		g_printf("malloc failed\n" );
		
    }
	
	/* obtain fingerprint image using library */
	
	result = fsi_init( &ctx );
	if( result < 0 )
    {
		/*adapted to glib */
		/*FIXME: use gtk dialog for this */
		g_printf( "%s\n", fsi_get_error_string(&ctx) );
		
    }
	
	/*adapted to glib*/
	/*g_printf( "swipe finger... " ); */
	gtk_label_set_text(evil_label, (gchar *) "swipe finger...");
	gtk_widget_show(evil_label);
	fflush( stdout );
	
	result = fsi_read_frames( &ctx, buffer, BUF_SIZE );
	if( result < 0 )
    {
		/* adapted to glib */
		/* FIXME: use gtk dialog for this */
		g_printf( "%s\n", fsi_get_error_string(&ctx) );
		
    }
	
	g_printf( "OK [dT=%d mK]\n", fsi_get_temperature(&ctx) );
	/*g_printf( "creating fingerprint image... " ); */
	
	gtk_label_set_text(evil_label, "creating fingerprint image... ");
	gtk_widget_show(evil_label);
	
	fflush( stdout );
	
	gettimeofday( &tv_start, NULL );
	result = fsi_merge_frames( &ctx );
	gettimeofday( &tv_now, NULL );
	if( result < 0 )
    {
		/* adapted to glib */
		/*FIXME: use gtk dialogs for this!*/
		g_printf( "%s\n", fsi_get_error_string(&ctx) );
	}
	num_lines = fsi_get_line_count( &ctx );
	
	line_width = fsi_get_line_width( &ctx );
	
	g_printf( "OK [size=%dx%d,Q=%d,dt=%ld]\n", line_width, num_lines, 
			  fsi_get_quality(&ctx), 
			  (tv_now.tv_sec-tv_start.tv_sec)*1000000 +
			  tv_now.tv_usec-tv_start.tv_usec );
	
	result = fsi_release( &ctx );
	if( result < 0 )
    {
		g_printf( "%s\n", fsi_get_error_string(&ctx) );
		
    }
	
	/* write image to file (pgm format) */
	
	buffer = realloc( buffer, num_lines * line_width ); /* trim buffer */
	if( buffer == NULL )
    {
		/* is this possible? (request is smaller than original size) */
		/* adapted to glib */
		/*FIXME: use dialogs for this*/
		g_printf( "realloc failed\n" );
	}
	/*adapted to glib, again  FIXME, dialog*/
	g_printf( "writing image to file... " ); fflush( stdout );
	
	
	
	fp = fopen( fname, "wb" );
	
	/* Dump it into a pixbuf FIXME: needs fixing*/
	/*GdkPixbuf*  fprintbuf = gdk_pixbuf_new_from_data (buffer, 
														GDK_COLORSPACE_RGB, 
														FALSE, 
														8, 
														line_width, 
														num_lines, 
														line_width, 
														NULL, 
														NULL);
	*/
	
	/*FIXME: write to memory or something and load from there*/
	write_pgm_file( buffer, line_width, num_lines, fp );
	
	g_printf( "OK [name=%s]\n", fname );
	fclose( fp );
	
	/* destroy buffer */
	
	free( buffer );
	
}




int main( int   argc,
          char *argv[] )
{
	/* create the window and packing boxes*/
	GtkWidget *window;
	GtkWidget *button_box;
	GtkWidget *image_box;		
	GtkWidget *v_box;	
	GtkWidget *scanbutton;
	GtkWidget *quitbutton;
	GtkWidget *scrolled_window;
	
	gtk_init (&argc, &argv);
	
	/* check number of arguments */ 
	if (argc != 2) {
		fprintf (stderr, "usage: gpe-fsi filename.pgm \n");
		/* This just does cleanup in GTK and exits with an exit status of 1. */
		/*exit (1);*/
	}
	
	
	/* initialize the window and packingboxes */
	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
	button_box = gtk_hbox_new (TRUE, 0);
	image_box = gtk_hbox_new (TRUE, 0);
	v_box = gtk_vbox_new (FALSE, 0);	
	
	evil_label = gtk_label_new("Welcome");
	
	/* the policy is one of GTK_POLICY AUTOMATIC, or GTK_POLICY_ALWAYS.
		* GTK_POLICY_AUTOMATIC will automatically decide whether you need
		* scrollbars, whereas GTK_POLICY_ALWAYS will always leave the scrollbars
		* there.  The first one is the horizontal scrollbar, the second, 
		* the vertical. */
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
									GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	
	/* You should always remember to connect the delete_event signal
		* to the main window. This is very important for proper intuitive
		* behavior */
	g_signal_connect (G_OBJECT (window), "delete_event",
					  G_CALLBACK (delete_event), NULL);
	
	
	/* pack the image_box into the vbox */
	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrolled_window), image_box);
	
	gtk_box_pack_start (GTK_BOX (v_box),scrolled_window, TRUE, TRUE, 0);
	
	/* pack the button_box into the v_box */
	gtk_box_pack_start (GTK_BOX (v_box), evil_label, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (v_box), button_box, FALSE, FALSE, 0);
	
	/* add v_box into the main window */
	gtk_container_add (GTK_CONTAINER (window), v_box);
	
	/* create the image widget, pack it into image_box and display it */
	img = gtk_image_new_from_file(argv[1]);
	gtk_box_pack_start (GTK_BOX (image_box), img, FALSE, FALSE, 0);
	gtk_widget_show(img);
	
	/* create the scanbutton and pack it into button_box */
	scanbutton = gtk_button_new_with_label ("Scan!");
	gtk_box_pack_start (GTK_BOX (button_box), scanbutton, FALSE, FALSE, 0);
	gtk_widget_show(scanbutton);	
	gtk_widget_show(evil_label);
	
	/* create the quitbutton and pack it into button_box */
	quitbutton = gtk_button_new_with_label ("quit");
	gtk_box_pack_start (GTK_BOX (button_box), quitbutton, FALSE, FALSE, 0);
	gtk_widget_show(quitbutton);
	
	
	/* Now when the button is clicked, we call the callback function */
	g_signal_connect (G_OBJECT (scanbutton), "clicked",
					  G_CALLBACK (scanfinger), (gpointer) "Scan!");
	g_signal_connect (G_OBJECT (scanbutton), "clicked",
					  G_CALLBACK (update_pic), (gpointer) fname);
	
	
	g_signal_connect (G_OBJECT (quitbutton), "clicked",
					  G_CALLBACK (delete_event), NULL);
	
	
	/* display the rest */
	gtk_widget_show (image_box);
	gtk_widget_show (scrolled_window);
	gtk_widget_show (button_box);
	gtk_widget_show (v_box);
	
	/* show the main window */
	gtk_widget_show  (window);
    
    gtk_main ();
    
    return 0;
}
